use super::macros::to_wasm;
use super::WasmAstNode;
use crate::error::CustomError;
use crate::types::Element;

pub trait ToWasm<'p> {
    type Output;

    fn to_wasm(&'p self, full_document: ToWasmContext<'_>) -> Result<Self::Output, CustomError>;
}

pub(crate) trait ToWasmStandardProperties {
    type Output;

    fn to_wasm_standard_properties(
        &self,
        wasm_context: ToWasmContext<'_>,
    ) -> Result<Self::Output, CustomError>;
}

#[derive(Debug, Clone)]
pub struct ToWasmContext<'s> {
    pub(crate) full_document: &'s str,
}

impl<'s> ToWasmContext<'s> {
    pub fn new(full_document: &'s str) -> ToWasmContext<'s> {
        ToWasmContext { full_document }
    }
}

to_wasm!(
    WasmAstNode<'s, 'p>,
    Element<'s>,
    original,
    wasm_context,
    standard_properties,
    {
        match original {
            Element::Paragraph(inner) => inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into),
            Element::PlainList(inner) => inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into),
            Element::CenterBlock(inner) => {
                inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into)
            }
            Element::QuoteBlock(inner) => {
                inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into)
            }
            Element::SpecialBlock(inner) => {
                inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into)
            }
            Element::DynamicBlock(inner) => {
                inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into)
            }
            Element::FootnoteDefinition(inner) => {
                inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into)
            }
            Element::Comment(inner) => inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into),
            Element::Drawer(inner) => inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into),
            Element::PropertyDrawer(inner) => {
                inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into)
            }
            Element::Table(inner) => inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into),
            Element::VerseBlock(inner) => {
                inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into)
            }
            Element::CommentBlock(inner) => {
                inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into)
            }
            Element::ExampleBlock(inner) => {
                inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into)
            }
            Element::ExportBlock(inner) => {
                inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into)
            }
            Element::SrcBlock(inner) => inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into),
            Element::Clock(inner) => inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into),
            Element::DiarySexp(inner) => inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into),
            Element::Planning(inner) => inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into),
            Element::FixedWidthArea(inner) => {
                inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into)
            }
            Element::HorizontalRule(inner) => {
                inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into)
            }
            Element::Keyword(inner) => inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into),
            Element::BabelCall(inner) => inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into),
            Element::LatexEnvironment(inner) => {
                inner.to_wasm(wasm_context).map(Into::<WasmAstNode>::into)
            }
        }
    }
);
